/* utilHash.h: general definitions for the hash table utility
 *
 * Modified Jul 24, 97 to add a function to return the hash key
 */

#ifndef UTILHASH_H
#define UTILHASH_H

#ifdef __cplusplus
#define ARGS(X) X
#define EXTERNC extern "C"
#else
#define ARGS(X) ()
#define EXTERNC
#endif


#ifndef __cplusplus
#include <sys/time.h>
#endif

#include <stdio.h>

#define HASH_DELETED ((char *) -1)

typedef char *itemPtr;

struct hashHeader
{
    char *(*keyFunc)(); /* gets the key given an itemPtr */
    char *(*mallocFunc)(); /* special function (optional) to be used for allocating memory */
    void (*freeFunc)(); /* special function (optional) to be used for freeing memory */
    int size; /* size of elements (below) */
    int numHashed; /* the number of items actually stored */
    int numUsed; /* count of items stored and deleted */
    int shouldFree; /* 1 if an element should be freed when deleted */
    itemPtr *elements; /* array of items that can be resized */
};

typedef struct hashHeader *hashTable;


/* -------- PUBLIC INTERFACE FUNCTIONS -------- */
EXTERNC hashTable createHashTable ARGS((
int initSize,
char *(*initKeyFunc)(char *),
int shouldFree
));

EXTERNC hashTable createHashTableSpecialMem ARGS((
int initSize,
char *(*initKeyFunc)(char *),
int shouldFree,
char *(*mallocFunc)(int),
void (*freeFunc)(void *)
));

EXTERNC itemPtr addToHashTable ARGS((
hashTable theTable,
itemPtr item
));

EXTERNC itemPtr findInHashTable ARGS((
hashTable theTable,
char *searchString
));

EXTERNC int deleteFromHashTable ARGS((
hashTable theTable,
itemPtr item
));

EXTERNC void destroyHashTable ARGS((
hashTable theTable
));

EXTERNC int hashTableDo ARGS((
hashTable theTable,
void (*aFunc)(char *, char *), /* takes an *item as arg1 and an arbitrary arg */
char *extraArg /* arbitrary second arg to be passed to aFunc */
));


EXTERNC int sizeOfHashTable ARGS((
hashTable theTable
));

EXTERNC int numHashedInHashTable ARGS((
hashTable theTable
));

EXTERNC float dispersion ARGS((
hashTable theTable
));

EXTERNC int hashValueOfString ARGS((
char *searchString
));  


/* ----------- PRIVATE ------------ */

char *testGetString();
char *randomString();
itemPtr *privateFindAux();
itemPtr privateInsert();
void printString();

/* the following can be over ridden by -D compiler directives. They take effect
when createHashTableSpecialMem() is called with NULL arguments */
#ifndef DEFAULT_MALLOC
char *allocFileMem();
#define DEFAULT_MALLOC allocFileMem
#endif

#ifndef DEFAULT_FREE
void freeFileMem();
#define DEFAULT_FREE freeFileMem
#endif

#ifndef DEFAULT_KEYFUNC
char *hashRecKeyFunc();
#define DEFAULT_KEYFUNC hashRecKeyFunc
#endif



#endif
