/*
 * utilHashAux.c: Utility functions called my functions in utilHash.c
 * None of these should be called directly.
 */

#include "utilHash.h"

/* ******** initialize the elements of a table; returns true if successful */
int privateInitHashElements(newTable, size)
hashTable newTable;
int size;
  {
    int i;
    itemPtr *elements;
    char *(*realMallocFunc)();

    realMallocFunc = (newTable->mallocFunc == NULL) ? DEFAULT_MALLOC : newTable->mallocFunc;

    elements = (itemPtr *)(realMallocFunc(size * sizeof(itemPtr)));
    if (elements == NULL) return(0);

    newTable->elements = elements;
    for (i=0; i<size; i++)
      {
        *(elements++) = NULL;
      }
    newTable->size = size;
    return(1);
  }

/* ******** calculate the has value of a string */
int privateHashVal(string, repeatCount, size)
char *string;
int repeatCount, size;
  {
    int max;
    char *c = string;
    unsigned int val = 7;
    int count = 1;

    max = 7 + repeatCount;
    while(*c != '\0' && count < max)
      {
        val = (val * (*c + repeatCount)) + 7927;
        c++;
        count ++;
      }
    return(val % size);
}

/* ******* find the value in the table
 *  Returns the address of the item if found (so it can be deleted)
 *  Returns a pointer to the first available spot otherwise.
 *  Returns NULL if we can't find a spot (this is an error).
 */
itemPtr *privateFindAux(theTable, searchString, iteration, firstSpot, lastPos)
hashTable theTable;
char *searchString;
int iteration;
int lastPos;
itemPtr *firstSpot; /* the first deleted spot we encountered */
  {
    itemPtr *eltAddr, elt;
    int pos;
    char *(*realKeyFunc)();

    realKeyFunc = (theTable->keyFunc == NULL) ? DEFAULT_KEYFUNC : theTable->keyFunc;
    
    /* check to see if we have searched the whole table */
    if(iteration > theTable->numUsed+3) return(NULL);
    
    /* calculate hash and look up the value */
    if(iteration < 4)
      pos = privateHashVal(searchString, iteration, theTable->size);
    else /* use sequential resolution on iterations 4 and above */
      pos = (lastPos + 1) % theTable->size;

    eltAddr = &((theTable->elements)[pos]);
    elt = *eltAddr;

    /* return if we are sure there is no item? */
    if(elt == NULL)
      {
        if(*firstSpot == NULL) return(eltAddr);
        else return(firstSpot);
      }

    /* if we have hit a deleted item -- keep looking, but keep a record of this */
    if(elt == HASH_DELETED)
      {
        if(*firstSpot == NULL) firstSpot = eltAddr;
      }
    else /* must be a value, but not necessarily matching */
      {
        if(!strcmp(realKeyFunc(elt), searchString))
          {
            return(eltAddr); /* found it */
          }
      }
    
    return(privateFindAux(theTable, searchString, iteration + 1, firstSpot, pos));
  }

/* ****** insert the value in the table - we know there is space
 * returns item if the item was already present or added; NULL on failure */
itemPtr privateInsert(theTable, item)
hashTable theTable;
itemPtr item;
  {
  itemPtr *addrToAdd, tempElement;
  char *(*realKeyFunc)();

  tempElement = NULL;

  realKeyFunc = (theTable->keyFunc == NULL) ? DEFAULT_KEYFUNC : theTable->keyFunc;

  addrToAdd = privateFindAux(theTable, realKeyFunc(item), 1, &tempElement, 0);

  /* error, could not find space */
  if(addrToAdd == NULL)
    {

      return(NULL);
    }

  if(*addrToAdd == NULL) theTable->numUsed++;

  if(*addrToAdd == NULL || *addrToAdd == HASH_DELETED)
    {
      *addrToAdd = item;
      theTable->numHashed ++;
    }

  return(*addrToAdd);
  }
